@@ -22,6 +22,7 @@
 """Generic utility helpers."""
 from __future__ import annotations
 
+import ast
 import contextlib
 import inspect
 import logging
@@ -38,6 +39,7 @@ from ansible.errors import AnsibleError, AnsibleParserError
 from ansible.module_utils.parsing.convert_bool import boolean
 from ansible.parsing.dataloader import DataLoader
 from ansible.parsing.mod_args import ModuleArgsParser
+from ansible.parsing.plugin_docs import read_docstring
 from ansible.parsing.yaml.constructor import AnsibleConstructor, AnsibleMapping
 from ansible.parsing.yaml.loader import AnsibleLoader
 from ansible.parsing.yaml.objects import AnsibleBaseYAMLObject, AnsibleSequence
@@ -186,7 +188,6 @@ def ansible_template(
                     _logger.warning(err)
                     raise
 
-                # pylint: disable=protected-access
                 templar.environment.filters._delegatee[  # noqa: SLF001
                     missing_filter
                 ] = mock_filter
@@ -566,7 +567,6 @@ def normalize_task_v2(task: dict[str, Any]) -> dict[str, Any]:
             skip_action_validation=options.skip_action_validation,
         )
     except AnsibleParserError as exc:
-        # pylint: disable=raise-missing-from
         raise MatchError(
             rule=AnsibleParserErrorRule(),
             message=exc.message,
@@ -755,7 +755,12 @@ class Task(dict[str, Any]):
 
     def is_handler(self) -> bool:
         """Return true for tasks that are handlers."""
-        return ".handlers[" in self.position
+        is_handler_file = False
+        if isinstance(self._normalized_task, dict):
+            file_name = str(self._normalized_task["action"].get(FILENAME_KEY, None))
+            if file_name:
+                is_handler_file = "handlers" in str(file_name)
+        return is_handler_file if is_handler_file else ".handlers[" in self.position
 
     def __repr__(self) -> str:
         """Return a string representation of the task."""
@@ -878,9 +883,7 @@ def parse_yaml_linenumbers(
         if hasattr(node, "__line__"):
             mapping[LINE_NUMBER_KEY] = node.__line__
         else:
-            mapping[
-                LINE_NUMBER_KEY
-            ] = mapping._line_number  # pylint: disable=protected-access  # noqa: SLF001
+            mapping[LINE_NUMBER_KEY] = mapping._line_number  # noqa: SLF001
         mapping[FILENAME_KEY] = lintable.path
         return mapping
 
@@ -983,7 +986,6 @@ def is_playbook(filename: str) -> bool:
     return False
 
 
-# pylint: disable=too-many-statements
 def get_lintables(
     opts: Options = options,
     args: list[str] | None = None,
@@ -1026,3 +1028,25 @@ def _extend_with_roles(lintables: list[Lintable]) -> None:
 def convert_to_boolean(value: Any) -> bool:
     """Use Ansible to convert something to a boolean."""
     return bool(boolean(value))
+
+
+def parse_examples_from_plugin(lintable: Lintable) -> tuple[int, str]:
+    """Parse yaml inside plugin EXAMPLES string.
+
+    Store a line number offset to realign returned line numbers later
+    """
+    offset = 1
+    parsed = ast.parse(lintable.content)
+    for child in parsed.body:
+        if isinstance(child, ast.Assign):
+            label = child.targets[0]
+            if isinstance(label, ast.Name) and label.id == "EXAMPLES":
+                offset = child.lineno - 1
+                break
+
+    docs = read_docstring(str(lintable.path))
+    examples = docs["plainexamples"]
+
+    # Ignore the leading newline and lack of document start
+    # as including those in EXAMPLES would be weird.
+    return offset, (f"---{examples}" if examples else "")
